# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class BillingFrequency(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The frequency with which the cost gets billed."""

    MONTHLY = "monthly"


class OperatorNumberType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of service associated with the phone number."""

    UNKNOWN = "unknown"
    OTHER = "other"
    GEOGRAPHIC = "geographic"
    MOBILE = "mobile"


class PhoneNumberAssignmentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Represents the assignment type of the offering."""

    PERSON = "person"
    APPLICATION = "application"


class PhoneNumberCapabilityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Capability value for calling."""

    NONE = "none"
    INBOUND = "inbound"
    OUTBOUND = "outbound"
    INBOUND_OUTBOUND = "inbound+outbound"


class PhoneNumberOperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of operation."""

    NOT_STARTED = "notStarted"
    RUNNING = "running"
    SUCCEEDED = "succeeded"
    FAILED = "failed"


class PhoneNumberOperationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of operation, e.g. Search."""

    PURCHASE = "purchase"
    RELEASE_PHONE_NUMBER = "releasePhoneNumber"
    SEARCH = "search"
    UPDATE_PHONE_NUMBER_CAPABILITIES = "updatePhoneNumberCapabilities"


class PhoneNumberSearchResultError(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Mapping Error Messages to Codes."""

    NO_ERROR = "NoError"
    UNKNOWN_ERROR_CODE = "UnknownErrorCode"
    OUT_OF_STOCK = "OutOfStock"
    AUTHORIZATION_DENIED = "AuthorizationDenied"
    MISSING_ADDRESS = "MissingAddress"
    INVALID_ADDRESS = "InvalidAddress"
    INVALID_OFFER_MODEL = "InvalidOfferModel"
    NOT_ENOUGH_LICENSES = "NotEnoughLicenses"
    NO_WALLET = "NoWallet"
    NOT_ENOUGH_CREDIT = "NotEnoughCredit"
    NUMBERS_PARTIALLY_ACQUIRED = "NumbersPartiallyAcquired"
    ALL_NUMBERS_NOT_ACQUIRED = "AllNumbersNotAcquired"
    RESERVATION_EXPIRED = "ReservationExpired"
    PURCHASE_FAILED = "PurchaseFailed"
    BILLING_UNAVAILABLE = "BillingUnavailable"
    PROVISIONING_FAILED = "ProvisioningFailed"
    UNKNOWN_SEARCH_ERROR = "UnknownSearchError"


class PhoneNumberType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Represents the number type of the offering."""

    GEOGRAPHIC = "geographic"
    TOLL_FREE = "tollFree"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.devtestlabs import DevTestLabsClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-devtestlabs
# USAGE
    python service_fabric_schedules_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DevTestLabsClient(
        credential=DefaultAzureCredential(),
        subscription_id="{subscriptionId}",
    )

    response = client.service_fabric_schedules.create_or_update(
        resource_group_name="resourceGroupName",
        lab_name="{labName}",
        user_name="@me",
        service_fabric_name="{serviceFrabicName}",
        name="{scheduleName}",
        schedule={
            "location": "{location}",
            "properties": {
                "dailyRecurrence": {"time": "19:00"},
                "hourlyRecurrence": {"minute": 0},
                "notificationSettings": {
                    "emailRecipient": "{email}",
                    "notificationLocale": "EN",
                    "status": "{Enabled|Disabled}",
                    "timeInMinutes": 15,
                    "webhookUrl": "{webhoolUrl}",
                },
                "status": "{Enabled|Disabled}",
                "targetResourceId": "/subscriptions/{subscriptionId}/resourceGroups/resourceGroupName/providers/Microsoft.DevTestLab/labs/{labName}/users/{uniqueIdentifier}/servicefabrics/{serviceFrabicName}",
                "taskType": "{Unknown|LabVmsShutdownTask|LabVmsStartupTask|LabVmReclamationTask|ComputeVmShutdownTask}",
                "timeZoneId": "Pacific Standard Time",
                "weeklyRecurrence": {
                    "time": "19:00",
                    "weekdays": ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"],
                },
            },
            "tags": {"tagName1": "tagValue1"},
        },
    )
    print(response)


# x-ms-original-file: specification/devtestlabs/resource-manager/Microsoft.DevTestLab/stable/2018-09-15/examples/ServiceFabricSchedules_CreateOrUpdate.json
if __name__ == "__main__":
    main()



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
from typing import Any, AsyncIterable, Callable, Dict, IO, Optional, TypeVar, Union, overload
import urllib.parse

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._scheduled_actions_operations import (
    build_check_name_availability_by_scope_request,
    build_check_name_availability_request,
    build_create_or_update_by_scope_request,
    build_create_or_update_request,
    build_delete_by_scope_request,
    build_delete_request,
    build_get_by_scope_request,
    build_get_request,
    build_list_by_scope_request,
    build_list_request,
    build_run_by_scope_request,
    build_run_request,
)

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class ScheduledActionsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.costmanagement.aio.CostManagementClient`'s
        :attr:`scheduled_actions` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(self, filter: Optional[str] = None, **kwargs: Any) -> AsyncIterable["_models.ScheduledAction"]:
        """List all private scheduled actions.

        .. seealso::
           - https://docs.microsoft.com/rest/api/cost-management/

        :param filter: May be used to filter scheduled actions by properties/viewId. Supported operator
         is 'eq'. Default value is None.
        :type filter: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either ScheduledAction or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.costmanagement.models.ScheduledAction]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ScheduledActionListResult] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_list_request(
                    filter=filter,
                    api_version=api_version,
                    template_url=self.list.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("ScheduledActionListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(
                    _models.ErrorResponseWithNestedDetails, pipeline_response
                )
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    list.metadata = {"url": "/providers/Microsoft.CostManagement/scheduledActions"}

    @distributed_trace
    def list_by_scope(
        self, scope: str, filter: Optional[str] = None, **kwargs: Any
    ) -> AsyncIterable["_models.ScheduledAction"]:
        """List all shared scheduled actions within the given scope.

        .. seealso::
           - https://docs.microsoft.com/rest/api/cost-management/

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param filter: May be used to filter scheduled actions by properties/viewId. Supported operator
         is 'eq'. Default value is None.
        :type filter: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either ScheduledAction or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.costmanagement.models.ScheduledAction]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ScheduledActionListResult] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_list_by_scope_request(
                    scope=scope,
                    filter=filter,
                    api_version=api_version,
                    template_url=self.list_by_scope.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("ScheduledActionListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(
                    _models.ErrorResponseWithNestedDetails, pipeline_response
                )
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    list_by_scope.metadata = {"url": "/{scope}/providers/Microsoft.CostManagement/scheduledActions"}

    @overload
    async def create_or_update(
        self,
        name: str,
        scheduled_action: _models.ScheduledAction,
        if_match: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ScheduledAction:
        """Create or update a private scheduled action.

        :param name: Scheduled action name. Required.
        :type name: str
        :param scheduled_action: Scheduled action to be created or updated. Required.
        :type scheduled_action: ~azure.mgmt.costmanagement.models.ScheduledAction
        :param if_match: ETag of the Entity. Not required when creating an entity. Optional when
         updating an entity and can be specified to achieve optimistic concurrency. Default value is
         None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update(
        self,
        name: str,
        scheduled_action: IO,
        if_match: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ScheduledAction:
        """Create or update a private scheduled action.

        :param name: Scheduled action name. Required.
        :type name: str
        :param scheduled_action: Scheduled action to be created or updated. Required.
        :type scheduled_action: IO
        :param if_match: ETag of the Entity. Not required when creating an entity. Optional when
         updating an entity and can be specified to achieve optimistic concurrency. Default value is
         None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update(
        self,
        name: str,
        scheduled_action: Union[_models.ScheduledAction, IO],
        if_match: Optional[str] = None,
        **kwargs: Any
    ) -> _models.ScheduledAction:
        """Create or update a private scheduled action.

        :param name: Scheduled action name. Required.
        :type name: str
        :param scheduled_action: Scheduled action to be created or updated. Is either a ScheduledAction
         type or a IO type. Required.
        :type scheduled_action: ~azure.mgmt.costmanagement.models.ScheduledAction or IO
        :param if_match: ETag of the Entity. Not required when creating an entity. Optional when
         updating an entity and can be specified to achieve optimistic concurrency. Default value is
         None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ScheduledAction] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(scheduled_action, (IOBase, bytes)):
            _content = scheduled_action
        else:
            _json = self._serialize.body(scheduled_action, "ScheduledAction")

        request = build_create_or_update_request(
            name=name,
            if_match=if_match,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.create_or_update.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize("ScheduledAction", pipeline_response)

        if response.status_code == 201:
            deserialized = self._deserialize("ScheduledAction", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    create_or_update.metadata = {"url": "/providers/Microsoft.CostManagement/scheduledActions/{name}"}

    @distributed_trace_async
    async def get(self, name: str, **kwargs: Any) -> _models.ScheduledAction:
        """Get the private scheduled action by name.

        .. seealso::
           - https://docs.microsoft.com/rest/api/cost-management/

        :param name: Scheduled action name. Required.
        :type name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ScheduledAction] = kwargs.pop("cls", None)

        request = build_get_request(
            name=name,
            api_version=api_version,
            template_url=self.get.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ScheduledAction", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {"url": "/providers/Microsoft.CostManagement/scheduledActions/{name}"}

    @distributed_trace_async
    async def delete(self, name: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """Delete a private scheduled action.

        .. seealso::
           - https://docs.microsoft.com/rest/api/cost-management/

        :param name: Scheduled action name. Required.
        :type name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_delete_request(
            name=name,
            api_version=api_version,
            template_url=self.delete.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    delete.metadata = {"url": "/providers/Microsoft.CostManagement/scheduledActions/{name}"}

    @overload
    async def create_or_update_by_scope(
        self,
        scope: str,
        name: str,
        scheduled_action: _models.ScheduledAction,
        if_match: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ScheduledAction:
        """Create or update a shared scheduled action within the given scope.

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param name: Scheduled action name. Required.
        :type name: str
        :param scheduled_action: Scheduled action to be created or updated. Required.
        :type scheduled_action: ~azure.mgmt.costmanagement.models.ScheduledAction
        :param if_match: ETag of the Entity. Not required when creating an entity. Optional when
         updating an entity and can be specified to achieve optimistic concurrency. Default value is
         None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update_by_scope(
        self,
        scope: str,
        name: str,
        scheduled_action: IO,
        if_match: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ScheduledAction:
        """Create or update a shared scheduled action within the given scope.

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param name: Scheduled action name. Required.
        :type name: str
        :param scheduled_action: Scheduled action to be created or updated. Required.
        :type scheduled_action: IO
        :param if_match: ETag of the Entity. Not required when creating an entity. Optional when
         updating an entity and can be specified to achieve optimistic concurrency. Default value is
         None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update_by_scope(
        self,
        scope: str,
        name: str,
        scheduled_action: Union[_models.ScheduledAction, IO],
        if_match: Optional[str] = None,
        **kwargs: Any
    ) -> _models.ScheduledAction:
        """Create or update a shared scheduled action within the given scope.

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param name: Scheduled action name. Required.
        :type name: str
        :param scheduled_action: Scheduled action to be created or updated. Is either a ScheduledAction
         type or a IO type. Required.
        :type scheduled_action: ~azure.mgmt.costmanagement.models.ScheduledAction or IO
        :param if_match: ETag of the Entity. Not required when creating an entity. Optional when
         updating an entity and can be specified to achieve optimistic concurrency. Default value is
         None.
        :type if_match: str
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ScheduledAction] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(scheduled_action, (IOBase, bytes)):
            _content = scheduled_action
        else:
            _json = self._serialize.body(scheduled_action, "ScheduledAction")

        request = build_create_or_update_by_scope_request(
            scope=scope,
            name=name,
            if_match=if_match,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.create_or_update_by_scope.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if response.status_code == 200:
            deserialized = self._deserialize("ScheduledAction", pipeline_response)

        if response.status_code == 201:
            deserialized = self._deserialize("ScheduledAction", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    create_or_update_by_scope.metadata = {"url": "/{scope}/providers/Microsoft.CostManagement/scheduledActions/{name}"}

    @distributed_trace_async
    async def get_by_scope(self, scope: str, name: str, **kwargs: Any) -> _models.ScheduledAction:
        """Get the shared scheduled action from the given scope by name.

        .. seealso::
           - https://docs.microsoft.com/rest/api/cost-management/

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param name: Scheduled action name. Required.
        :type name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ScheduledAction or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.ScheduledAction
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ScheduledAction] = kwargs.pop("cls", None)

        request = build_get_by_scope_request(
            scope=scope,
            name=name,
            api_version=api_version,
            template_url=self.get_by_scope.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ScheduledAction", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_by_scope.metadata = {"url": "/{scope}/providers/Microsoft.CostManagement/scheduledActions/{name}"}

    @distributed_trace_async
    async def delete_by_scope(  # pylint: disable=inconsistent-return-statements
        self, scope: str, name: str, **kwargs: Any
    ) -> None:
        """Delete a scheduled action within the given scope.

        .. seealso::
           - https://docs.microsoft.com/rest/api/cost-management/

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param name: Scheduled action name. Required.
        :type name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_delete_by_scope_request(
            scope=scope,
            name=name,
            api_version=api_version,
            template_url=self.delete_by_scope.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    delete_by_scope.metadata = {"url": "/{scope}/providers/Microsoft.CostManagement/scheduledActions/{name}"}

    @distributed_trace_async
    async def run(self, name: str, **kwargs: Any) -> None:  # pylint: disable=inconsistent-return-statements
        """Processes a private scheduled action.

        :param name: Scheduled action name. Required.
        :type name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_run_request(
            name=name,
            api_version=api_version,
            template_url=self.run.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    run.metadata = {"url": "/providers/Microsoft.CostManagement/scheduledActions/{name}/execute"}

    @distributed_trace_async
    async def run_by_scope(  # pylint: disable=inconsistent-return-statements
        self, scope: str, name: str, **kwargs: Any
    ) -> None:
        """Runs a shared scheduled action within the given scope.

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param name: Scheduled action name. Required.
        :type name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_run_by_scope_request(
            scope=scope,
            name=name,
            api_version=api_version,
            template_url=self.run_by_scope.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    run_by_scope.metadata = {"url": "/{scope}/providers/Microsoft.CostManagement/scheduledActions/{name}/execute"}

    @overload
    async def check_name_availability(
        self,
        check_name_availability_request: _models.CheckNameAvailabilityRequest,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Checks availability and correctness of the name for a scheduled action.

        :param check_name_availability_request: Scheduled action to be created or updated. Required.
        :type check_name_availability_request:
         ~azure.mgmt.costmanagement.models.CheckNameAvailabilityRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def check_name_availability(
        self, check_name_availability_request: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Checks availability and correctness of the name for a scheduled action.

        :param check_name_availability_request: Scheduled action to be created or updated. Required.
        :type check_name_availability_request: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def check_name_availability(
        self, check_name_availability_request: Union[_models.CheckNameAvailabilityRequest, IO], **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Checks availability and correctness of the name for a scheduled action.

        :param check_name_availability_request: Scheduled action to be created or updated. Is either a
         CheckNameAvailabilityRequest type or a IO type. Required.
        :type check_name_availability_request:
         ~azure.mgmt.costmanagement.models.CheckNameAvailabilityRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.CheckNameAvailabilityResponse] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(check_name_availability_request, (IOBase, bytes)):
            _content = check_name_availability_request
        else:
            _json = self._serialize.body(check_name_availability_request, "CheckNameAvailabilityRequest")

        request = build_check_name_availability_request(
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.check_name_availability.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CheckNameAvailabilityResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    check_name_availability.metadata = {"url": "/providers/Microsoft.CostManagement/checkNameAvailability"}

    @overload
    async def check_name_availability_by_scope(
        self,
        scope: str,
        check_name_availability_request: _models.CheckNameAvailabilityRequest,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Checks availability and correctness of the name for a scheduled action within the given scope.

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param check_name_availability_request: Scheduled action to be created or updated. Required.
        :type check_name_availability_request:
         ~azure.mgmt.costmanagement.models.CheckNameAvailabilityRequest
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def check_name_availability_by_scope(
        self, scope: str, check_name_availability_request: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Checks availability and correctness of the name for a scheduled action within the given scope.

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param check_name_availability_request: Scheduled action to be created or updated. Required.
        :type check_name_availability_request: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def check_name_availability_by_scope(
        self,
        scope: str,
        check_name_availability_request: Union[_models.CheckNameAvailabilityRequest, IO],
        **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Checks availability and correctness of the name for a scheduled action within the given scope.

        :param scope: The scope associated with scheduled action operations. This includes
         'subscriptions/{subscriptionId}' for subscription scope,
         'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}' for resourceGroup scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}' for Billing Account scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/departments/{departmentId}' for
         Department scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/enrollmentAccounts/{enrollmentAccountId}'
         for EnrollmentAccount scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/billingProfiles/{billingProfileId}'
         for BillingProfile scope,
         'providers/Microsoft.Billing/billingAccounts/{billingAccountId}/invoiceSections/{invoiceSectionId}'
         for InvoiceSection scope,
         'providers/Microsoft.CostManagement/externalBillingAccounts/{externalBillingAccountName}' for
         External Billing Account scope and
         'providers/Microsoft.CostManagement/externalSubscriptions/{externalSubscriptionName}' for
         External Subscription scope. Note: Insight Alerts are only available on subscription scope.
         Required.
        :type scope: str
        :param check_name_availability_request: Scheduled action to be created or updated. Is either a
         CheckNameAvailabilityRequest type or a IO type. Required.
        :type check_name_availability_request:
         ~azure.mgmt.costmanagement.models.CheckNameAvailabilityRequest or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.costmanagement.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.CheckNameAvailabilityResponse] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(check_name_availability_request, (IOBase, bytes)):
            _content = check_name_availability_request
        else:
            _json = self._serialize.body(check_name_availability_request, "CheckNameAvailabilityRequest")

        request = build_check_name_availability_by_scope_request(
            scope=scope,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.check_name_availability_by_scope.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseWithNestedDetails, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CheckNameAvailabilityResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    check_name_availability_by_scope.metadata = {
        "url": "/{scope}/providers/Microsoft.CostManagement/checkNameAvailability"
    }



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_agents_openapi_connection_auth.py

DESCRIPTION:
    This sample demonstrates how to use agent operations with the
    OpenAPI tool from the Azure Agents service using a synchronous client, using
    custom key authentication against the TripAdvisor API.
    To learn more about OpenAPI specs, visit https://learn.microsoft.com/openapi

USAGE:
    python sample_agents_openapi_connection_auth.py

    Before running the sample:

    Set up an account at https://www.tripadvisor.com/developers and get an API key.

    Set up a custom key connection and save the connection name following the steps at
    https://aka.ms/azsdk/azure-ai-agents/custom-key-setup

    Save that connection name as the PROJECT_OPENAPI_CONNECTION_NAME environment variable

    pip install azure-ai-agents azure-identity jsonref

    Set this environment variables with your own values:
    PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                       page of your Azure AI Foundry portal.
    OPENAPI_CONNECTION_ID - the connection ID for the OpenAPI connection, taken from Azure AI Foundry.
    MODEL_DEPLOYMENT_NAME - name of the model deployment in the project to use Agents against
"""

import os
import jsonref
from azure.ai.agents import AgentsClient
from azure.identity import DefaultAzureCredential
from azure.ai.agents.models import OpenApiTool, OpenApiConnectionAuthDetails, OpenApiConnectionSecurityScheme

asset_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../assets/tripadvisor_openapi.json"))

agents_client = AgentsClient(
    endpoint=os.environ["PROJECT_ENDPOINT"],
    credential=DefaultAzureCredential(),
)

model_name = os.environ["MODEL_DEPLOYMENT_NAME"]
connection_id = os.environ["OPENAPI_CONNECTION_ID"]

print(connection_id)

with open(asset_file_path, "r") as f:
    openapi_spec = jsonref.loads(f.read())

# Create Auth object for the OpenApiTool (note that connection or managed identity auth setup requires additional setup in Azure)
auth = OpenApiConnectionAuthDetails(security_scheme=OpenApiConnectionSecurityScheme(connection_id=connection_id))

# Initialize an Agent OpenApi tool using the read in OpenAPI spec
openapi = OpenApiTool(
    name="get_weather", spec=openapi_spec, description="Retrieve weather information for a location", auth=auth
)

# Create an Agent with OpenApi tool and process Agent run
with agents_client:
    agent = agents_client.create_agent(
        model=model_name, name="my-agent", instructions="You are a helpful agent", tools=openapi.definitions
    )
    print(f"Created agent, ID: {agent.id}")

    # Create thread for communication
    thread = agents_client.threads.create()
    print(f"Created thread, ID: {thread.id}")

    # Create message to thread
    message = agents_client.messages.create(
        thread_id=thread.id,
        role="user",
        content="Summarize the reviews for the top rated hotel in Paris",
    )
    print(f"Created message: {message['id']}")

    # Create and process an Agent run in thread with tools
    run = agents_client.runs.create_and_process(thread_id=thread.id, agent_id=agent.id)
    print(f"Run finished with status: {run.status}")

    if run.status == "failed":
        print(f"Run failed: {run.last_error}")

    # Delete the Agent when done
    agents_client.delete_agent(agent.id)
    print("Deleted agent")

    # Fetch and log all messages
    messages = agents_client.messages.list(thread_id=thread.id)
    for msg in messages:
        if msg.text_messages:
            last_text = msg.text_messages[-1]
            print(f"{msg.role}: {last_text.text.value}")



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import pytest
import datetime
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.serialization import _datetime_as_isostr  # pylint:disable=protected-access
from _router_test_case_async import AsyncRouterRecordedTestCase
from _validators import RouterJobValidator
from azure.communication.jobrouter._model_base import _deserialize_datetime as _convert_str_to_datetime
from _decorators_async import RouterPreparersAsync
from azure.communication.jobrouter._shared.utils import parse_connection_str
from azure.core.exceptions import ResourceNotFoundError

from azure.communication.jobrouter.aio import (
    JobRouterClient,
    JobRouterAdministrationClient,
)
from azure.communication.jobrouter.models import (
    RoundRobinMode,
    RouterWorkerSelector,
    LabelOperator,
    RouterQueueSelector,
    StaticQueueSelectorAttachment,
    StaticRouterRule,
    StaticWorkerSelectorAttachment,
    RouterJobStatus,
    DistributionPolicy,
    RouterQueue,
    ClassificationPolicy,
    RouterJob,
    ScheduleAndSuspendMode,
    RouterJobNote,
)

job_labels = {"key1": "JobKey", "key2": 10, "key3": True}

job_tags = {
    "tag1": "WorkerGenericInfo",
}

job_channel_references = ["fakeChannelRef1", "fakeChannelRef2"]

job_channel_ids = ["fakeChannelId1", "fakeChannelId2"]

job_priority = 10

job_disposition_code = "JobCancelledByUser"

job_requested_worker_selectors = [
    RouterWorkerSelector(key="FakeKey1", label_operator=LabelOperator.EQUAL, value=True),
    RouterWorkerSelector(key="FakeKey2", label_operator=LabelOperator.NOT_EQUAL, value=False),
    RouterWorkerSelector(key="FakeKey3", label_operator=LabelOperator.LESS_THAN, value=10),
    RouterWorkerSelector(key="FakeKey4", label_operator=LabelOperator.LESS_THAN_OR_EQUAL, value=10.01),
    RouterWorkerSelector(key="FakeKey5", label_operator=LabelOperator.GREATER_THAN, value=10),
    RouterWorkerSelector(key="FakeKey6", label_operator=LabelOperator.GREATER_THAN_OR_EQUAL, value=10),
]


prioritization_rules = [
    StaticRouterRule(value=10),
]

cp_worker_selectors = [
    StaticWorkerSelectorAttachment(
        worker_selector=RouterWorkerSelector(
            key="FakeKeyFromCp",
            label_operator=LabelOperator.EQUAL,
            value="FakeValue",
        )
    ),
]

expected_attached_worker_selectors = [
    RouterWorkerSelector(
        key="FakeKeyFromCp",
        label_operator=LabelOperator.EQUAL,
        value="FakeValue",
    ),
]

test_timestamp = _convert_str_to_datetime("2022-05-13T23:59:04.5311999+07:00")
job_notes = [RouterJobNote(added_at=test_timestamp, message="Fake notes attached to job")]


# The test class name needs to start with "Test" to get collected by pytest
class TestRouterJobAsync(AsyncRouterRecordedTestCase):
    async def clean_up(self):
        # delete in live mode
        if not self.is_playback():
            router_client: JobRouterClient = self.create_client()
            router_admin_client: JobRouterAdministrationClient = self.create_admin_client()
            async with router_client:
                async with router_admin_client:
                    if self._testMethodName in self.job_ids and any(self.job_ids[self._testMethodName]):
                        for _id in set(self.job_ids[self._testMethodName]):
                            await self.clean_up_job(job_id=_id)

                    if self._testMethodName in self.classification_policy_ids and any(
                        self.classification_policy_ids[self._testMethodName]
                    ):
                        for policy_id in set(self.classification_policy_ids[self._testMethodName]):
                            await router_admin_client.delete_classification_policy(policy_id)

                    if self._testMethodName in self.queue_ids and any(self.queue_ids[self._testMethodName]):
                        for _id in set(self.queue_ids[self._testMethodName]):
                            await router_admin_client.delete_queue(_id)

                    if self._testMethodName in self.distribution_policy_ids and any(
                        self.distribution_policy_ids[self._testMethodName]
                    ):
                        for policy_id in set(self.distribution_policy_ids[self._testMethodName]):
                            await router_admin_client.delete_distribution_policy(policy_id)

    def get_distribution_policy_id(self):
        return "_" + self._testMethodName + "_tst_dp_async"

    async def setup_distribution_policy(self):
        client: JobRouterAdministrationClient = self.create_admin_client()

        async with client:
            distribution_policy_id = self.get_distribution_policy_id()

            policy: DistributionPolicy = DistributionPolicy(
                name=distribution_policy_id,
                offer_expires_after_seconds=10.0,
                mode=RoundRobinMode(min_concurrent_offers=1, max_concurrent_offers=1),
            )

            distribution_policy = await client.upsert_distribution_policy(distribution_policy_id, policy)

            # add for cleanup later
            if self._testMethodName in self.distribution_policy_ids:
                self.distribution_policy_ids[self._testMethodName] = self.distribution_policy_ids[
                    self._testMethodName
                ].append(distribution_policy_id)
            else:
                self.distribution_policy_ids[self._testMethodName] = [distribution_policy_id]

    def get_job_queue_id(self):
        return "_" + self._testMethodName + "_tst_q_async"

    async def setup_job_queue(self):
        client: JobRouterAdministrationClient = self.create_admin_client()

        async with client:
            job_queue_id = self.get_job_queue_id()

            job_queue: RouterQueue = RouterQueue(
                name=job_queue_id, labels=job_labels, distribution_policy_id=self.get_distribution_policy_id()
            )

            job_queue = await client.upsert_queue(job_queue_id, job_queue)

            # add for cleanup later
            if self._testMethodName in self.queue_ids:
                self.queue_ids[self._testMethodName].append(job_queue_id)
            else:
                self.queue_ids[self._testMethodName] = [job_queue_id]

    def get_fallback_queue_id(self):
        return "_" + self._testMethodName + "_tst_flbk_q_async"  # cspell:disable-line

    async def setup_fallback_queue(self):
        client: JobRouterAdministrationClient = self.create_admin_client()

        async with client:
            job_queue_id = self.get_fallback_queue_id()

            job_queue: RouterQueue = RouterQueue(
                name=job_queue_id, labels=job_labels, distribution_policy_id=self.get_distribution_policy_id()
            )

            job_queue = await client.upsert_queue(job_queue_id, job_queue)

            # add for cleanup later
            if self._testMethodName in self.queue_ids:
                self.queue_ids[self._testMethodName].append(job_queue_id)
            else:
                self.queue_ids[self._testMethodName] = [job_queue_id]

    def get_classification_policy_id(self):
        return "_" + self._testMethodName + "_tst_cp"

    async def setup_classification_policy(self):
        client: JobRouterAdministrationClient = self.create_admin_client()

        async with client:
            cp_queue_selectors = [
                StaticQueueSelectorAttachment(
                    queue_selector=RouterQueueSelector(
                        key="Id", label_operator=LabelOperator.EQUAL, value=self.get_job_queue_id()
                    )
                ),
            ]

            cp_id = self.get_classification_policy_id()

            classification_policy: ClassificationPolicy = ClassificationPolicy(
                name=cp_id,
                fallback_queue_id=self.get_fallback_queue_id(),
                queue_selector_attachments=cp_queue_selectors,
                prioritization_rule=prioritization_rules[0],
                worker_selector_attachments=cp_worker_selectors,
            )

            cp = await client.upsert_classification_policy(cp_id, classification_policy)

            # add for cleanup later
            if self._testMethodName in self.classification_policy_ids:
                self.classification_policy_ids[self._testMethodName].append(cp_id)
            else:
                self.classification_policy_ids[self._testMethodName] = [cp_id]

    async def validate_job_is_queued(
        self,
        identifier,  # type: str
        **kwargs,  # type: Any
    ):
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job = await router_client.get_job(identifier)
            assert router_job.status == RouterJobStatus.QUEUED

    async def validate_job_is_scheduled(self, identifier, **kwargs):
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job = await router_client.get_job(identifier)
            assert router_job.status == RouterJobStatus.SCHEDULED

    async def validate_job_is_cancelled(self, identifier, **kwargs):
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job = await router_client.get_job(identifier)
            assert router_job.status == RouterJobStatus.CANCELLED

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_create_job_direct_q(self):
        job_identifier = "tst_create_job_man_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert router_job.status == RouterJobStatus.CREATED

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_create_scheduled_job(self, **kwargs):
        recorded_variables = kwargs.pop("variables", {})
        scheduled_time = datetime.datetime.utcnow() + datetime.timedelta(0, 30)
        scheduled_time_utc = recorded_variables.setdefault("scheduled_time_utc", _datetime_as_isostr(scheduled_time))

        matching_mode = ScheduleAndSuspendMode(schedule_at=recorded_variables["scheduled_time_utc"])

        job_identifier = "_tst_create_sch_job_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_id=job_channel_ids[0],
                channel_reference=job_channel_references[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
                matching_mode=matching_mode,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
                matching_mode=matching_mode,
            )

            assert router_job.status == RouterJobStatus.PENDING_SCHEDULE

            await self._poll_until_no_exception(self.validate_job_is_scheduled, Exception, job_identifier)

            return recorded_variables

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_update_job_direct_q(self):
        job_identifier = "tst_update_job_man_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

            # Act
            router_job.labels["FakeKey"] = "FakeWorkerValue"
            updated_job_labels = router_job.labels

            update_router_job = await router_client.upsert_job(job_identifier, router_job)

            assert update_router_job is not None
            RouterJobValidator.validate_job(
                update_router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=updated_job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            # updating labels does not change job status
            assert update_router_job.status == RouterJobStatus.QUEUED

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_update_job_direct_q_w_kwargs(self):
        job_identifier = "tst_update_job_man_w_kwargs_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

            # Act
            updated_job_labels = {k: v for k, v in router_job.labels.items()}
            updated_job_labels["FakeKey"] = "FakeWorkerValue"

            update_router_job = await router_client.upsert_job(job_identifier, labels=updated_job_labels)

            assert update_router_job is not None
            RouterJobValidator.validate_job(
                update_router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=updated_job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            # updating labels does not change job status
            assert update_router_job.status == RouterJobStatus.QUEUED

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_get_job_direct_q(self):
        job_identifier = "tst_get_job_man_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert router_job.status == RouterJobStatus.CREATED

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

            queried_router_job = await router_client.get_job(job_identifier)

            RouterJobValidator.validate_job(
                queried_router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_fallback_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_classification_policy")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_create_job_w_cp(self):
        job_identifier = "tst_create_job_cp_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert router_job.status == RouterJobStatus.PENDING_CLASSIFICATION

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_fallback_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_classification_policy")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_update_job_w_cp(self):
        job_identifier = "tst_update_job_cp_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert router_job.status == RouterJobStatus.PENDING_CLASSIFICATION

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

            # Act
            updated_job_labels = {k: v for k, v in router_job.labels.items()}
            updated_job_labels["FakeKey"] = "FakeWorkerValue"
            router_job.labels = updated_job_labels

            update_router_job = await router_client.upsert_job(job_identifier, router_job)

            assert update_router_job is not None
            RouterJobValidator.validate_job(
                update_router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=updated_job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            # updating labels reverts job status to pending classification
            assert update_router_job.status == RouterJobStatus.PENDING_CLASSIFICATION

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_fallback_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_classification_policy")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_update_job_w_cp_w_kwargs(self):
        job_identifier = "tst_update_job_cp_w_kwargs_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert router_job.status == RouterJobStatus.PENDING_CLASSIFICATION

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

            # Act
            updated_job_labels = {k: v for k, v in router_job.labels.items()}
            updated_job_labels["FakeKey"] = "FakeWorkerValue"

            update_router_job = await router_client.upsert_job(job_identifier, labels=updated_job_labels)

            assert update_router_job is not None
            RouterJobValidator.validate_job(
                update_router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=updated_job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            # updating labels reverts job status to pending classification
            assert update_router_job.status == RouterJobStatus.PENDING_CLASSIFICATION

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_fallback_queue")
    @RouterPreparersAsync.before_test_execute_async("setup_classification_policy")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_get_job_w_cp(self):
        job_identifier = "tst_get_job_cp_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                classification_policy_id=self.get_classification_policy_id(),
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert router_job.status == RouterJobStatus.PENDING_CLASSIFICATION

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

            queried_router_job = await router_client.get_job(job_identifier)

            RouterJobValidator.validate_job(
                queried_router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                attached_worker_selectors=expected_attached_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert queried_router_job.status == RouterJobStatus.QUEUED

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_delete_job(self):
        job_identifier = "_tst_del_job_man_async"
        router_client: JobRouterClient = self.create_client()

        async with router_client:
            router_job: RouterJob = RouterJob(
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            router_job = await router_client.upsert_job(job_identifier, router_job)

            # add for cleanup
            self.job_ids[self._testMethodName] = [job_identifier]

            assert router_job is not None
            RouterJobValidator.validate_job(
                router_job,
                identifier=job_identifier,
                channel_reference=job_channel_references[0],
                channel_id=job_channel_ids[0],
                queue_id=self.get_job_queue_id(),
                priority=job_priority,
                requested_worker_selectors=job_requested_worker_selectors,
                labels=job_labels,
                tags=job_tags,
                notes=job_notes,
            )

            assert router_job.status == RouterJobStatus.CREATED

            await self._poll_until_no_exception(self.validate_job_is_queued, Exception, job_identifier)

            # job needs to be in a termination state before it can be deleted
            await router_client.cancel_job(job_identifier)
            await router_client.delete_job(job_identifier)
            with pytest.raises(ResourceNotFoundError) as nfe:
                await router_client.get_job(job_identifier)
                self.job_ids.pop(self._testMethodName, None)
            assert nfe.value.reason == "Not Found"
            assert nfe.value.status_code == 404

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_list_jobs(self):
        router_client: JobRouterClient = self.create_client()
        job_identifiers = ["tst_list_job_1_async", "tst_list_job_2_async", "tst_list_job_3_async"]

        created_job_response = {}
        job_count = len(job_identifiers)
        self.job_ids[self._testMethodName] = []

        async with router_client:
            for identifier in job_identifiers:
                router_job: RouterJob = RouterJob(
                    channel_reference=job_channel_references[0],
                    channel_id=job_channel_ids[0],
                    queue_id=self.get_job_queue_id(),
                    priority=job_priority,
                    requested_worker_selectors=job_requested_worker_selectors,
                    labels=job_labels,
                    tags=job_tags,
                    notes=job_notes,
                )

                router_job = await router_client.upsert_job(identifier, router_job)

                # add for cleanup
                self.job_ids[self._testMethodName].append(identifier)

                assert router_job is not None

                RouterJobValidator.validate_job(
                    router_job,
                    identifier=identifier,
                    channel_reference=job_channel_references[0],
                    channel_id=job_channel_ids[0],
                    queue_id=self.get_job_queue_id(),
                    priority=job_priority,
                    requested_worker_selectors=job_requested_worker_selectors,
                    labels=job_labels,
                    tags=job_tags,
                    notes=job_notes,
                )

                await self._poll_until_no_exception(self.validate_job_is_queued, Exception, identifier)

                created_job_response[router_job.id] = router_job

            router_jobs = router_client.list_jobs(
                results_per_page=3,
                status=RouterJobStatus.QUEUED,
                queue_id=self.get_job_queue_id(),
                channel_id=job_channel_ids[0],
            )

            async for router_job_page in router_jobs.by_page():
                list_of_jobs = [i async for i in router_job_page]
                assert len(list_of_jobs) <= 3

                for j_item in list_of_jobs:
                    response_at_creation = created_job_response.get(j_item.id, None)

                    if not response_at_creation:
                        continue

                    RouterJobValidator.validate_job(
                        j_item,
                        identifier=response_at_creation.id,
                        channel_reference=response_at_creation.channel_reference,
                        channel_id=response_at_creation.channel_id,
                        queue_id=response_at_creation.queue_id,
                        priority=response_at_creation.priority,
                        requested_worker_selectors=response_at_creation.requested_worker_selectors,
                        labels=response_at_creation.labels,
                        tags=response_at_creation.tags,
                        notes=response_at_creation.notes,
                    )
                    job_count -= 1

            # all job_queues created were listed
            assert job_count == 0

    @RouterPreparersAsync.router_test_decorator_async
    @recorded_by_proxy_async
    @RouterPreparersAsync.before_test_execute_async("setup_distribution_policy")
    @RouterPreparersAsync.before_test_execute_async("setup_job_queue")
    @RouterPreparersAsync.after_test_execute_async("clean_up")
    async def test_list_sch_jobs(self, **kwargs):
        recorded_variables = kwargs.pop("variables", {})
        scheduled_time = datetime.datetime.utcnow() + datetime.timedelta(0, 30)
        scheduled_time_utc = recorded_variables.setdefault("scheduled_time_utc", _datetime_as_isostr(scheduled_time))

        matching_mode = ScheduleAndSuspendMode(schedule_at=recorded_variables["scheduled_time_utc"])

        router_client: JobRouterClient = self.create_client()
        job_identifiers = ["_tst_list_sch_job_1_async", "_tst_list_sch_job_2_async"]

        created_job_response = {}
        job_count = len(job_identifiers)
        self.job_ids[self._testMethodName] = []

        async with router_client:
            for identifier in job_identifiers:
                router_job: RouterJob = RouterJob(
                    channel_reference=job_channel_references[0],
                    channel_id=job_channel_ids[0],
                    queue_id=self.get_job_queue_id(),
                    priority=job_priority,
                    requested_worker_selectors=job_requested_worker_selectors,
                    labels=job_labels,
                    tags=job_tags,
                    notes=job_notes,
                    matching_mode=matching_mode,
                )

                router_job = await router_client.upsert_job(identifier, router_job)

                # add for cleanup
                self.job_ids[self._testMethodName].append(identifier)

                assert router_job is not None

                RouterJobValidator.validate_job(
                    router_job,
                    identifier=identifier,
                    channel_reference=job_channel_references[0],
                    channel_id=job_channel_ids[0],
                    queue_id=self.get_job_queue_id(),
                    priority=job_priority,
                    requested_worker_selectors=job_requested_worker_selectors,
                    labels=job_labels,
                    tags=job_tags,
                    notes=job_notes,
                    matching_mode=matching_mode,
                )

                await self._poll_until_no_exception(self.validate_job_is_scheduled, Exception, identifier)

                created_job_response[router_job.id] = router_job

            router_jobs = router_client.list_jobs(
                results_per_page=2,
                status=RouterJobStatus.SCHEDULED,
                queue_id=self.get_job_queue_id(),
                scheduled_before=recorded_variables["scheduled_time_utc"],
                channel_id=job_channel_ids[0],
            )

            async for router_job_page in router_jobs.by_page():
                list_of_jobs = [i async for i in router_job_page]
                assert len(list_of_jobs) <= 2

                for j_item in list_of_jobs:
                    response_at_creation = created_job_response.get(j_item.id, None)

                    if not response_at_creation:
                        continue

                    RouterJobValidator.validate_job(
                        j_item,
                        identifier=response_at_creation.id,
                        channel_reference=response_at_creation.channel_reference,
                        channel_id=response_at_creation.channel_id,
                        queue_id=response_at_creation.queue_id,
                        priority=response_at_creation.priority,
                        requested_worker_selectors=response_at_creation.requested_worker_selectors,
                        labels=response_at_creation.labels,
                        tags=response_at_creation.tags,
                        notes=response_at_creation.notes,
                        matching_mode=response_at_creation.matching_mode,
                    )
                    job_count -= 1

            # all job_queues created were listed
            assert job_count == 0
            return recorded_variables
